#!/usr/bin/python
"""Eep RDF API: Built-Ins"""

import eep, query, infer
import copy

nslist, stuff = [], {}

# Register the namespaces

def regns(ns): 
   if ns not in nslist: nslist.append(ns)

LOG_NS = 'http://www.w3.org/2000/10/swap/log#'
STRING_NS = 'http://www.w3.org/2000/10/swap/string#'
UTIL_NS = 'http://example.org/#'
regns(LOG_NS)
regns(STRING_NS)
regns(UTIL_NS)

# Create the builtins

class BI_log_notEqualTo:
    def evaluate(self, t): 
       return (t[0].repr != t[2].repr)

class BI_log_uri:
    def evaluateObject(self, t): 
       if t[0].type == 'URI': 
          t = t[:]
          return [t[0], t[1], eep.Article('"'+t[0].val+'"')]
       else: return 1

class BI_log_content:
    def evaluateObject(self, t): 
       import urllib2
       if t[0].type == 'URI': 
          t = t[:]
          try: content = urllib2.urlopen(t[0].val).read()
          except: content = 'Error getting URI'
          return [t[0], t[1], eep.Article('"'+content+'"')]
       else: return 1

class BI_string_notEqualTo:
    def evaluate(self, t): 
       if (t[0].type != 'Literal') or (t[2].type != 'Literal'): return 0
       else: return (t[0].val != t[2].val)

class BI_util_sha512: 
   def evaluateObject(self, t): 
      import sha512
      if t[0].type == 'Literal': 
         t = t[:]
         t2 = eep.Article('"'+sha512.new(t[0].val).hexdigest()+'"')
         return [t[0], t[1], t2]
      else: return 1

# Register the builtins

def register(c, ns, name): 
   stuff[ns+name] = c

register(BI_log_notEqualTo, LOG_NS, 'notEqualTo')
register(BI_log_uri, LOG_NS, 'uri')
register(BI_log_content, LOG_NS, 'content')
register(BI_string_notEqualTo, STRING_NS, 'notEqualTo')
register(BI_util_sha512, UTIL_NS, 'sha512')

# Other stuff

def isBI(t): 
   """Tests for a triple using a built-in predicate"""
   if t[1].type != 'URI': return 0
   do = 0
   for ns in nslist: 
      if t[1].val[:len(ns)] == ns: do = 1
   if not do: return 0
   if t[1].val not in stuff.keys(): return 0
   return 1

def bi(t): 
   """This is optimized so that it shouldn't slow down queries too much."""
   if (t[0].type == 'Univar') and (t[2].type == 'Univar'): 
      return t
   elif (t[0].type == 'Univar') and (t[2].type != 'Univar'): 
      return stuff[t[1].val]().evaluateSubject(t)
   elif (t[0].type != 'Univar') and (t[2].type == 'Univar'): 
      return stuff[t[1].val]().evaluateObject(t)
   elif (t[0].type != 'Univar') and (t[2].type != 'Univar'): 
      if stuff[t[1].val]().evaluate(t): return 1
      else: return 0

def bfilter(r, store): 
   desc = r[1]
   ante = filter(lambda t: not isBI(t), r[0])
   bins = filter(lambda t: isBI(t), r[0])

   # Make a dictionary of the vars in the builtins
   bd = {}
   for triple in bins: 
      for pos in (0, 1, 2): 
         if triple[pos].type[-3:] == 'var': bd[triple[pos].repr] = None
   qres, result = query.xtquery(ante, store), []
   for res in qres: 
      newbins, bdict, int = [triple[:] for triple in bins], copy.copy(bd), []
      for b in newbins: 
         for pos in (0, 1, 2): 
            if (b[pos].type == 'Univar') and (b[pos].repr in res[1].keys()): 
               b[pos] = eep.Article(res[1][b[pos].repr])
      for b in newbins: 
         biResult = bi(b)
         if biResult == 0: int.append(0)
         if biResult != 1: 
            for pos in (0, 1, 2): 
               if b[pos].type == 'Univar': 
                  bdict[b[pos].repr] = biResult[pos]
      for t in desc: 
         t = t[:]
         for p in (0, 1, 2): 
            if (t[p].type == 'Univar') and (t[p].repr in res[1].keys()): 
               t[p] = eep.Article(res[1][t[p].repr])
            elif (t[p].type == 'Univar') and (t[p].repr in bdict.keys()): 
               t[p] = eep.Article(bdict[t[p].repr])
         int.append(t)
      if len(int) > 0: 
         if int[0] != 0: result.append(int)
      else: result.append(int)
   return result

if __name__=="__main__": 
   print __doc__